/**************************************************************************
*  Copyright (c) 1997-1998 by Joseph Parrello. All Rights Reserved.       *
*  Site   : http://space.tin.it/computer/giparrel                         *
*  E-Mail : joparrello@geocities.com, joparrel@tin.it                     *
*                                                                         *
*  Revision:   2.0  (August 18, 1998).                                    *
*                                                                         *
*  Definition of JPMRURegistry class.                                     *
*                                                                         *
*  Read JPMRUREG.TXT for use and license.                                 *
*                                                                         *
*  Only for OWL 5.02 and 5.01 users.                                      *
*  OWL and ObjectWindows are trademarks of Borland International.         *
*                                                                         *
*  THIS CLASS AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF     *
*  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO    *
*  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A         *
*  PARTICULAR PURPOSE.                                                    *
**************************************************************************/
#include <owl/pch.h>

#if !defined(OWL_WINDOW_RH)
# include <owl/window.rh>
#endif
#if !defined(CLASSLIB_FILENAME_H)
# include <classlib\filename.h>
#endif

#include "jpmrureg.h"


/*************************************************************************************
*   Build a response table for all messages/commands handled by the application.     *
*************************************************************************************/
DEFINE_RESPONSE_TABLE1(JPMRURegistry, TEventHandler)
//{{JPMRURegistryRSP_TBL_BEGIN}}
  EV_COMMAND_AND_ID(CM_MRU_FIRST + 0, CmSelect),
  EV_COMMAND_AND_ID(CM_MRU_FIRST + 1, CmSelect),
  EV_COMMAND_AND_ID(CM_MRU_FIRST + 2, CmSelect),
  EV_COMMAND_AND_ID(CM_MRU_FIRST + 3, CmSelect),
  EV_COMMAND_AND_ID(CM_MRU_FIRST + 4, CmSelect),
  EV_COMMAND_AND_ID(CM_MRU_FIRST + 5, CmSelect),
  EV_COMMAND_AND_ID(CM_MRU_FIRST + 6, CmSelect),
  EV_COMMAND_AND_ID(CM_MRU_FIRST + 7, CmSelect),
  EV_COMMAND_AND_ID(CM_MRU_FIRST + 8, CmSelect),
  EV_COMMAND_AND_ID(CM_MRU_FIRST + 9, CmSelect),
  EV_COMMAND_ENABLE(CM_EXIT, CeExit),
//{{JPMRURegistryRSP_TBL_END}}
END_RESPONSE_TABLE;


//{{JPMRURegistry Implementation}}
const _TCHAR* MRUKey32       = "Recent File List";  // for compatibility with MFC
const _TCHAR* MRUKey16       = "Recent.File.List";  // Win 3.1 doesn't support spaces in registry
const _TCHAR* MruPrefix      = "File";              // for compatibility with MFC
const _TCHAR* MRUKeyDefault  = "NoKey";             // For no key found
const int   MaxMenuItemLen   = 255;                 // for compatibility with TRecentFiles class
#if defined(BI_PLAT_WIN16)
#define HKEY_CURRENT_USER ((HKEY) 0x80000001L)    // defined only for 16 bit platforms
const int   MaxRegValueLen = 80;                  // for Win16 target applications
#else
const int   MaxRegValueLen = MaxMenuItemLen;      // 256 is defined in \INCLUDE\WIN32\REGSTR.H
#endif                                            // Here is a limit due to Menu's Item Length
uint JPMRURegistry::MRUMessage = 0;

/***********************************************************************************
*   Constructor to initialize the key on registry and to read the MRU List.        *
***********************************************************************************/
JPMRURegistry::JPMRURegistry(const _TCHAR far* regKey,
                             uint MRUsize,
                             bool IsCurDir,
                             bool IsUnderExitMenu,
                             bool ForceUpdate,
                             bool CheckFileExistence,
                             bool NoExtDisplay,
                             int  MaxItemLen)
: MRUList(MaxMenuItems + 1,0,0)
{
  // INSERT>> Your code here.
   checkfileExistence = CheckFileExistence;
   ForceRegistryUpdate = ForceUpdate;
   MRUUnderExitMenu = IsUnderExitMenu;
   TrackCurrentDir = IsCurDir;
   MaxMenuItemLength = MaxItemLen;
   noextDisplay = NoExtDisplay;
   StoredHMenu = NULL;
   RegistryKey = RootKey = NULL;
   IsNewSeparator = false;
   CurrentExitPosition = 0;
   SetMaxMruItems(MRUsize, false);
   bool win32Bit = true;
#if defined(BI_PLAT_WIN16)
   DWORD dwVersion = ::GetVersion();
   if ((LOBYTE(LOWORD(dwVersion)) == 3) && (HIBYTE(LOWORD(dwVersion)) < 95))
      win32Bit = false;
#endif
   if (win32Bit)
#if defined(BI_PLAT_WIN16)
     CreateRegistryKey(HKEY_CURRENT_USER, regKey, MRUKey16);
#else
     CreateRegistryKey(HKEY_CURRENT_USER, regKey, MRUKey32);
#endif
   else
     CreateRegistryKey(HKEY_CLASSES_ROOT, regKey, MRUKey16);
   MRUMessage = ::RegisterWindowMessage(MruFileMessage);
   ReadMRU();
}

/***********************************************************************************
*   Write the MRU on registry and free the handle of key (if exists).              *
***********************************************************************************/
JPMRURegistry::~JPMRURegistry()
{
  // INSERT>> Your code here.
   WriteMRU();
   if (RegistryKey)
      ::RegCloseKey(RegistryKey);
}

/***********************************************************************************
*   Set the max number of items in MRU and delete the remaining items from it.     *
*   Provided for compatibility with TRecentFiles::SetMaxMruItems function.         *
***********************************************************************************/
void JPMRURegistry::SetMaxMruItems(int max, bool update)
{
   MaxItemsInMRU = min(max, (int) MaxMenuItems);
   DeleteScraps();
   if (ForceRegistryUpdate && update)  // update the registry now if "ForceUpdate" option is on.
      WriteMRU();
}

/***********************************************************************************
*   Get the text of the item based on menu ID.                                     *
*   Provided for compatibility with TRecentFiles::GetMenuText function.            *
***********************************************************************************/
bool JPMRURegistry::GetMenuText(int id, _TCHAR far* text, int maxTextLen)
{
  id -= CM_MRU_FIRST;
  if (id >= 0 && id <= GetMRUSize())
    {
    _tcsncpy(text, MRUList[id].c_str(), maxTextLen);
    return true;
    }
  return false;
}

/***********************************************************************************
*   Save the menu item on the MRU list.                                            *
*   Provided for compatibility with TRecentFiles::SaveMenuChoice function.         *
***********************************************************************************/
void JPMRURegistry::SaveMenuChoice(const _TCHAR far* text)
{
   int index = FindTextInArray(text);
   RemoveStringFromList(index);
   AddStringToList(text);
}

/********************************************************************************
*   If user has selected an item in the MRU list.                               *
********************************************************************************/
void JPMRURegistry::CmSelect(uint idMenu)
{
   TApplication* theApp = TYPESAFE_DOWNCAST(this, TApplication);
   if (theApp)
      {
      TFrameWindow* mainWin = theApp->GetMainWindow();
      if (mainWin)
         mainWin->SendMessage(MRUMessage, idMenu, 0);
      }
}

/***********************************************************************************
*   The sheet-anchor of this class. Obtains the menu handle to display MRU List.   *
***********************************************************************************/
void JPMRURegistry::CeExit(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  TMenuItemEnabler* menuEnabler = TYPESAFE_DOWNCAST(&tce, TMenuItemEnabler);
  if (menuEnabler)
     UpdateMenu(menuEnabler);
  tce.Enable(true);
}

/***********************************************************************************
*   Retrieve the current size of MRU List.                                         *
***********************************************************************************/
int JPMRURegistry::GetMRUSize()
{
  return MRUList.GetItemsInContainer();
}

/***********************************************************************************
*   Fill the MRU List reading from the registry.                                   *
***********************************************************************************/
void JPMRURegistry::ReadMRU()
{
  if (RegistryKey)
     {
     _TCHAR keyBuf [MaxRegValueLen], tempBuf [MaxRegValueLen];
     MRUList.Flush();       // Removes all strings from array (for precaution).
     for (uint base = 1; base <= MaxItemsInMRU; base++)
        {
        wsprintf(keyBuf, _T("%s%d"), MruPrefix, base);
        GetRegistryStringValue(RegistryKey, keyBuf, tempBuf, MaxMenuItemLen, MRUKeyDefault);
        if ((_tcslen(tempBuf) == 0) || (_tcscmp(tempBuf, MRUKeyDefault) == 0))
           continue;                   // no more keys found
        if (checkfileExistence)
           {
           OFSTRUCT ofs;               // check if file exists
           ofs.cBytes = sizeof ofs;
           if (::OpenFile(tempBuf, &ofs, OF_EXIST) == -1)
              continue;
           }
        MRUList.Add(tempBuf);  // add the key to array
        }
     if (ForceRegistryUpdate)  // update the registry now if "ForceUpdate" option is on.
         WriteMRU();
     }
}

/***********************************************************************************
*   Write the MRU List on the registry.                                            *
***********************************************************************************/
void JPMRURegistry::WriteMRU()
{
   if (RegistryKey)
      {
      _TCHAR keyBuf [MaxRegValueLen];
      int index = GetMRUSize();
      for (int i = 0; i < MaxMenuItems; i++)
         {
         wsprintf(keyBuf, _T("%s%d"), MruPrefix, i + 1);
         if (i < index)
            SetRegistryStringValue(RegistryKey, keyBuf, MRUList[i].c_str());
         else
            DeleteRegKey(RegistryKey, keyBuf);
         }
      }
}

/***********************************************************************************
*   Find a string in the MRU List (case-insensitive search).                       *
***********************************************************************************/
int JPMRURegistry::FindTextInArray(const _TCHAR far* text)
{
  int isFound = -1;
  for (int i = 0; i < GetMRUSize(); i++)
     {
     if (_tcscmpi(MRUList[i].c_str(), text) == 0)
        {
        isFound = i;
        break;
        }
     }
  return isFound;
}

/***********************************************************************************
*   Add a string to MRU List and delete the unnecessary ones.                      *
***********************************************************************************/
void JPMRURegistry::AddStringToList(const _TCHAR far* text)
{
   MRUList.AddAt(text, 0);
   DeleteScraps();
   if (ForceRegistryUpdate)
      WriteMRU();
}

/***********************************************************************************
*   Remove a string from the MRU List based on index.                              *
***********************************************************************************/
void JPMRURegistry::RemoveStringFromList(int index)
{
  if (index >= 0 && index <= GetMRUSize())
     MRUList.RemoveEntry(index);
}

/***********************************************************************************
*   Delete unnecessary strings from the MRU List.                                  *
***********************************************************************************/
void JPMRURegistry::DeleteScraps()
{
   uint index = GetMRUSize();
   while (index > MaxItemsInMRU)
      {
      MRUList.RemoveEntry(index - 1);
      index = GetMRUSize();
      }
}

/***********************************************************************************
*   Insert the MRU List in "File" menu.                                            *
***********************************************************************************/
void JPMRURegistry::RetrieveMenuItemString(LPTSTR lpszFileName)
{
 // If the following option is enabled, we delete the extension.
 if (noextDisplay)
    {
	 for (int i = _tcslen(lpszFileName) - 1; i--; i >= 0)
        {
        if (lpszFileName[i] == '.')
           {
           lpszFileName[i] = '\0';
           break;
           }
        }
    }
// If the following option is enabled, we return only the filename if the
// file is in the current directory.
 if (TrackCurrentDir)
    {
    TFileName currentDir(TFileName::CurrentDir);   // store current directory
    TFileName tempFile = lpszFileName;     // store current file information
    string curFile = tempFile.GetParts(TFileName::Server | TFileName::Device | TFileName::Path);
    string curDir = currentDir.GetParts(TFileName::Server | TFileName::Device);
    if ((currentDir.HasParts(TFileName::Path) == false) && (currentDir.HasParts(TFileName::File)))
      curDir += '\\';
    curDir += currentDir.GetParts(TFileName::Path | TFileName::File);
    if (currentDir.HasParts(TFileName::File))
       curDir += '\\';
    if (curDir == curFile)  // if file is in current directory retrieve only filename
       {
       lpszFileName[0] = '\0';
       _tcscat(lpszFileName, tempFile.GetParts(TFileName::File | TFileName::Ext).c_str());
       }
    }
// If the following option is enabled, we shrink the fullpathname in order to limit its length.
 if ((MaxMenuItemLength > 0) && (MaxMenuItemLength < (int)_tcslen(lpszFileName)))
    {
    int nName;
    const _TCHAR* curString;
    const _TCHAR* startString;
    TFileName tempFile = TFileName(lpszFileName);
    nName = _tcslen(tempFile.GetParts(TFileName::File | TFileName::Ext).c_str());
    string pathstr = tempFile.GetParts(TFileName::Path);
    uint maxsize = _tcslen(pathstr.c_str());
    if (maxsize > 0)
       {
       if (pathstr[maxsize - 1] == '\\')
          {
          maxsize--;
          pathstr.resize(maxsize);
          }
       bool alright = false;
       startString = pathstr.c_str();
       curString = pathstr.c_str();
       uint remsize = nName + 4 + (_tcslen(tempFile.GetParts(TFileName::Device).c_str()) + 1) + (_tcslen(tempFile.GetParts(TFileName::Server).c_str()));
       uint i = 0;
       while ((alright == false) && (i < maxsize))
           {
           if ((startString[i] == '\\') && (i > 0))
              {
              if ((int) (maxsize - i + remsize) <= MaxMenuItemLength)
                 {
                 alright = true;
                 break;
                 }
              }
           curString = _tcsinc(curString);
           i++;
           }
       string fnameString = _T("\\...");
       if (alright)
          fnameString += (string) curString;
       tempFile.SetPart(TFileName::Path, fnameString);
       lpszFileName[0] = '\0';
       _tcscat(lpszFileName, tempFile.GetParts(TFileName::Server | TFileName::Device | TFileName::Path | TFileName::File | TFileName::Ext).c_str());
       }
    }
}

/***********************************************************************************
*   Insert the MRU List in "File" menu.                                            *
***********************************************************************************/
void JPMRURegistry::UpdateMenu(TMenuItemEnabler* menuEnabler)
{
  HMENU hMenu = menuEnabler->GetMenu();
  int exitMenuPos = menuEnabler->GetPosition();
  int mrusize = GetMRUSize();
  // if the menu is new, update the internal variables.
  if (StoredHMenu != hMenu)
     {
     CurrentExitPosition = exitMenuPos;
     StoredHMenu = hMenu;
     IsNewSeparator = false;
     }
  // remove all MRU's menu items.
  for (int i = ::GetMenuItemCount(hMenu); i >= 0; i--)
      {
      uint id = ::GetMenuItemID(hMenu, i);
      if ((id >= CM_MRU_FIRST) && (id < CM_MRU_FIRST + MaxMenuItems))
         ::RemoveMenu(hMenu, i, MF_BYPOSITION);
      }
  // remove the separator only if there is no item in the list.
  if ((mrusize == 0) && (IsNewSeparator) && (exitMenuPos > 1))
     {
     bool result;
     if (MRUUnderExitMenu)
         result = (bool) ::RemoveMenu(hMenu, exitMenuPos + 1, MF_BYPOSITION);
     else
         result = (bool) ::RemoveMenu(hMenu, exitMenuPos - 1, MF_BYPOSITION);
     if (result)
        IsNewSeparator = false;
     }
  // add an item menu for each item of the list.
  uint mruCount = 0;
  TMenu MRUMenu(hMenu);
  for (int i = 0; i < mrusize; i++)
       {
       _TCHAR tempBuf[MaxMenuItemLen], menuBuf[MaxMenuItemLen];
       wsprintf(tempBuf, _T("%s"), MRUList[i].c_str());
       RetrieveMenuItemString(tempBuf);
       if (i >= 9)
         wsprintf(menuBuf, _T("1&%d  %s"), i - 9,  tempBuf);
       else      // put an ampersand in number '0' (number '1' has already an ampersand)
         wsprintf(menuBuf, _T("&%d    %s"), i + 1, tempBuf);
       bool result;
       if (MRUUnderExitMenu)
          result = (bool) MRUMenu.AppendMenu(MF_STRING, CM_MRU_FIRST + i, menuBuf);
       else
          result = (bool) MRUMenu.InsertMenu(CurrentExitPosition + i, MF_BYPOSITION | MF_STRING, CM_MRU_FIRST + i, menuBuf);
       if (result)
          mruCount++;
       }
   if  (mruCount > 0)
       {
       // Insert a separator only if it doesn't exist.
       if (!MRUUnderExitMenu)
          {
          if (::GetMenuItemID(hMenu, CurrentExitPosition + mruCount) != 0)
             IsNewSeparator = (bool) MRUMenu.InsertMenu(CurrentExitPosition + mruCount, MF_BYPOSITION | MF_SEPARATOR, 0, NULL);
          }
       else
          {
          if (::GetMenuItemID(hMenu, CurrentExitPosition + 1) != 0)
             IsNewSeparator = (bool) MRUMenu.InsertMenu(CurrentExitPosition + 1, MF_BYPOSITION | MF_SEPARATOR, 0, NULL);
          }
       }
}

/***********************************************************************************
*   Allows you to track the current directory on MRU List. Default is false.       *
***********************************************************************************/
void JPMRURegistry::EnableDirectoryFeature(bool enable)
{
   TrackCurrentDir = enable;
}

bool JPMRURegistry::GetDirectoryFeature()
{
   return TrackCurrentDir;
}

/***********************************************************************************
*   Allows you to put the MRU List under Exit item. Default is false.              *
***********************************************************************************/
void JPMRURegistry::PutMRUUnderExitMenu(bool enable)
{
   MRUUnderExitMenu = enable;
}

bool JPMRURegistry::IsMRUUnderExitMenu()
{
   return MRUUnderExitMenu;
}

/***********************************************************************************
*   Allows you to force registry update when you select a file from the MRU.       *
*   Default is false.                                                              *
***********************************************************************************/
void JPMRURegistry::SetMRUUpdate(bool enable)
{
   ForceRegistryUpdate = enable;
}

bool JPMRURegistry::GetMRUUpdate()
{
   return ForceRegistryUpdate;
}

/***********************************************************************************
*   Allows you to check if MRU's files exist. If they exist, will be added         *
*   to the MRU List.  Default is false.                                            *
***********************************************************************************/
void JPMRURegistry::SetMRUFileExistence(bool enable)
{
   checkfileExistence = enable;
}

bool JPMRURegistry::GetMRUFileExistence()
{
   return checkfileExistence;
}

/***********************************************************************************
*   Allows you to have the file displayed without its extension.                   *
*   Default is false.                                                              *
***********************************************************************************/
void JPMRURegistry::SetNoExtDisplay(bool enable)
{
   noextDisplay = enable;
}

bool JPMRURegistry::GetNoExtDisplay()
{
   return noextDisplay;
}

/***********************************************************************************
*   Allows you to set the maximum length for each menu item.                       *
*   Default is 0.                                                                  *
***********************************************************************************/
void JPMRURegistry::SetMaxItemLength(int value)
{
   if (value >= 0)
      MaxMenuItemLength = value;
   else
      MaxMenuItemLength = 0;
}

int JPMRURegistry::GetMaxItemLength()
{
   return MaxMenuItemLength;
}

/***********************************************************************************
*   Retrieve the root key on the registry.                                         *
***********************************************************************************/
HKEY JPMRURegistry::GetRegistryRootKey()
{
   return RootKey;
}

/***********************************************************************************
*   Create (or open if exists) the subkey of a given key.                          *
***********************************************************************************/
HKEY JPMRURegistry::CreateRegistrySubkey(HKEY Key, const _TCHAR far* buf)
{
   HKEY pKey = NULL;
#if defined(BI_PLAT_WIN16)
   long result = ::RegCreateKey(Key, buf, &pKey);
#else
   uint32  dwDisposition;
   long result = ::RegCreateKeyEx(Key, buf, 0, "", REG_OPTION_NON_VOLATILE,
                                    KEY_ALL_ACCESS, NULL, &pKey, &dwDisposition);
#endif
   if (result != ERROR_SUCCESS)
      pKey = NULL;
   return pKey;
}

/***********************************************************************************
*   Create (or open if exists) the main key on the registry.                       *
***********************************************************************************/
void JPMRURegistry::CreateRegistryKey(HKEY Key, const _TCHAR far* keyName, const _TCHAR far* subkeyName)
{
   _TCHAR buf[MaxMenuItemLen];
   wsprintf(buf, _T("Software\\%s"), keyName);
   RootKey = CreateRegistrySubkey(Key, buf);
   if (RootKey)
      RegistryKey = CreateRegistrySubkey(RootKey, subkeyName);
}

/***********************************************************************************
*   Read a string value from the registry.                                         *
***********************************************************************************/
bool JPMRURegistry::GetRegistryStringValue(HKEY regkey, const _TCHAR far* subkeyName,
                                           _TCHAR far* buff, unsigned buffSize,
                                           const _TCHAR far* defaultString)
{
   bool result = true;
   buff[0] = '\0';
#if defined(BI_PLAT_WIN16)
   long value = ::RegQueryValue(regkey, subkeyName, (LPSTR)buff, (long*)(LPDWORD) &buffSize);
#else
   DWORD dwType = REG_SZ;
   long value = ::RegQueryValueEx(regkey, subkeyName, 0, &dwType, (LPBYTE) buff, (uint32*) &buffSize);
#endif
   if (value!=ERROR_SUCCESS)
      {
      if (!defaultString)
         result = false;
      else _tcscpy(buff, defaultString);
      }
   return result;
}

/***********************************************************************************
*   Write a string value on the registry.                                          *
***********************************************************************************/
bool JPMRURegistry::SetRegistryStringValue(HKEY regkey, const _TCHAR* key, const _TCHAR* str)
{
DWORD size = _tcslen(str);
#if defined(BI_PLAT_WIN16)
  long result = ::RegSetValue(regkey, key, REG_SZ, (LPCSTR)(uint8*) str, size);
#else
  long result = ::RegSetValueEx(regkey, key, 0, REG_SZ, (uint8*) str, size);
#endif
  return (bool) (result == ERROR_SUCCESS);
}

/***********************************************************************************
*   Read a numeric value from the registry.                                        *
***********************************************************************************/
int JPMRURegistry::GetRegistryIntegerValue(HKEY regkey, const _TCHAR far* subkeyName, int defvalue)
{
    int  retValue = defvalue;
#if defined(BI_PLAT_WIN16)
    _TCHAR valuebuf[20];
    bool result = GetRegistryStringValue(regkey, subkeyName, valuebuf, sizeof(valuebuf), "");
    if (result && (_tcslen(valuebuf) > 0))
       retValue = _ttoi(valuebuf);
#elif defined(BI_PLAT_WIN32)
    DWORD dwValue;
    DWORD dwType = REG_DWORD;
    DWORD dwCount = sizeof(DWORD);
    long value = ::RegQueryValueEx(regkey, subkeyName, 0, &dwType, (LPBYTE)&dwValue, &dwCount);
    if (value == ERROR_SUCCESS)
       retValue = (int) dwValue;
#endif
    return retValue;
}

/***********************************************************************************
*   Write a numeric value on the registry.                                         *
***********************************************************************************/
bool JPMRURegistry::SetRegistryIntegerValue(HKEY regkey, const _TCHAR* key, int value)
{
   bool result;
#if defined(BI_PLAT_WIN16)
   _TCHAR valuebuf[20];
   wsprintf(valuebuf, _T("%i"), value);
   result = (bool) SetRegistryStringValue(regkey, key, valuebuf);
#elif defined(BI_PLAT_WIN32)
   long lResult = ::RegSetValueEx(regkey, key, 0, REG_DWORD, (LPBYTE)&value, sizeof(value));
   result = (bool) (lResult == ERROR_SUCCESS);
#endif
   return result;
}

/***********************************************************************************
*   Delete a value from the registry.                                              *
***********************************************************************************/
bool JPMRURegistry::DeleteRegKey(HKEY regkey, const _TCHAR* key)
{
#if defined(BI_PLAT_WIN16)
  long result = ::RegDeleteKey(regkey, key);
#else
  long result = ::RegDeleteValue(regkey, key);
#endif
  return (bool) (result == ERROR_SUCCESS);
}

