/**************************************************************************
*  Copyright (c) 1997-1998 by Joseph Parrello. All Rights Reserved.       *
*  Site   : http://space.tin.it/computer/giparrel                         *
*  E-Mail : joparrello@geocities.com, joparrel@tin.it                     *
*                                                                         *
*  Revision:   2.0  (August 18, 1998).                                    *
*                                                                         *
*  Definition of JPMRURegistry class.                                     *
*                                                                         *
*  Read JPMRUREG.TXT for use and license.                                 *
*                                                                         *
*  Only for OWL 5.02 and 5.01 users.                                      *
*  OWL and ObjectWindows are trademarks of Borland International.         *
*                                                                         *
*  THIS CLASS AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF     *
*  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO    *
*  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A         *
*  PARTICULAR PURPOSE.                                                    *
**************************************************************************/
#if !defined(jpmrureg_h)              // Sentry, use file only if it's not already included.
#define jpmrureg_h

#if !defined(OWL_EVENTHAN_H)
# include <owl/eventhan.h>
#endif
#if !defined( CLASSLIB_ARRAYS_H )
# include <classlib/arrays.h>
#endif
#if !defined(__TCHAR_H)
# include <tchar.h>
#endif

#define MruFileMessage  "MRUFILEMESSAGE"  // for compatibility with TRecentFiles class.
const int CM_MRU_FIRST  =  31990;         // for compatibility with TRecentFiles class.

//{{TEventHandler = JPMRURegistry}}
class JPMRURegistry : virtual public TEventHandler {
  public:
    JPMRURegistry(const _TCHAR far* regKey,
                  uint MRUsize = MaxMenuItems,
                  bool IsCurDir = false,
                  bool IsUnderExitMenu = false,
                  bool ForceUpdate = false,
                  bool CheckFileExistence = false,
                  bool NoExtDisplay = false,
                  int  MaxItemLen = 0);
    virtual ~JPMRURegistry();

    // for compatibility with TRecentFiles class
  public:
    enum          { MaxMenuItems = 10 };
    void          SetMaxMruItems(int max, bool update = true);
    bool          GetMenuText(int id, _TCHAR far* text, int maxTextLen);
    void          SaveMenuChoice(const _TCHAR far* text);
    //  new public functions to enable or disable the new features of this class.
    void          EnableDirectoryFeature(bool enable);
    bool          GetDirectoryFeature();
    void          PutMRUUnderExitMenu(bool enable);
    bool          IsMRUUnderExitMenu();
    void          SetMRUUpdate(bool enable);
    bool          GetMRUUpdate();
    void          SetMRUFileExistence(bool enable);
    bool          GetMRUFileExistence();
    void          SetNoExtDisplay(bool enable);
    bool          GetNoExtDisplay();
    void          SetMaxItemLength(int value);
    int           GetMaxItemLength();
    bool          GetRegistryStringValue(HKEY regkey, const _TCHAR far* subkeyName,
                                         _TCHAR far* buff, unsigned buffSize,
                                         const _TCHAR far* defaultString);
    bool          SetRegistryStringValue(HKEY regkey, const _TCHAR* key, const _TCHAR* str);
    int           GetRegistryIntegerValue(HKEY regkey, const _TCHAR far* subkeyName, int defvalue = 0);
    bool          SetRegistryIntegerValue(HKEY regkey, const _TCHAR* key, int value);
    bool          DeleteRegKey(HKEY regkey, const _TCHAR* key);
    HKEY          CreateRegistrySubkey(HKEY Key, const _TCHAR far* buf);
    HKEY          GetRegistryRootKey();
    void          RetrieveMenuItemString(LPTSTR lpszFileName);

//{{JPMRURegistryRSP_TBL_BEGIN}}
  protected:
    void    CmSelect(uint idMenu);
    void    CeExit(TCommandEnabler &tce);
    int     GetMRUSize();
    void    ReadMRU();
    void    WriteMRU();
    int     FindTextInArray(const _TCHAR far* text);
    void    RemoveStringFromList(int index);
    void    DeleteScraps();
    void    AddStringToList(const _TCHAR far* text);
    void    UpdateMenu(TMenuItemEnabler* menuEnabler);
    void    CreateRegistryKey(HKEY Key, const _TCHAR far* keyName, const _TCHAR far* subkeyName);
    uint    MRUGetFileName(LPCTSTR lpszPathName, LPTSTR lpszTitle, UINT nMax);
//{{JPMRURegistryRSP_TBL_END}}

  private:
    void          Read();
    static uint   MRUMessage;
    uint          MaxItemsInMRU;
    TArrayAsVector<string> MRUList;
    bool          IsNewSeparator;
    HMENU         StoredHMenu;
    bool          TrackCurrentDir;
    bool          MRUUnderExitMenu;
    HKEY          RegistryKey, RootKey;
    bool          ForceRegistryUpdate;
    bool          checkfileExistence;
    bool          noextDisplay;
    int           CurrentExitPosition;
    int           MaxMenuItemLength;

DECLARE_RESPONSE_TABLE(JPMRURegistry);
};    //{{JPMRURegistry}}

const int CM_MRU_LAST  =  CM_MRU_FIRST + JPMRURegistry::MaxMenuItems;

#endif  // jpmrureg_h sentry.

